; FREAXx.S - Alien shootem-up game. (C) A.Millett 2014-2024. 
;
;  Released as free software under GNU GPL3 license, see GNU website:    www.gnu.org/licenses/gpl-3.0.html 
;----------------------------------------------------------------------------
; FREAX is a simple Alien shootem-up game that compiles for the VIC20, C64 & C16/+4.
;  To compile under CA65 set the USE_CA65 flag.
;  For notes and info on compiling, see FREAX.TXT
;----------------------------------------------------------------------------
; MODLOG 
;  Simple init chrset, show alien chr.
;-> FREAX1.S (5.7.14)
;  Imp 4-phase 12 bit wide alien chr.
;  Imp NewGame to init AlienLo/AlienHi/AlienType[MAXALIEN] arrays.
;  Imp DrawScreen fn to draw..
;-> FREAX2.S (7.7.14)
;  Imp simple anim
;-> FREAX3.S (8.7.14)
;  Change to using AlienX/Y co-oords in 1/4 chrs (ie: [0-159,0-24] on 40x25 VDU)
;  Simpler anim calcs, bound checks etc.
;  Simple anim working approx as previously.
;-> FREAX4.S (10.7.14)
;  Imp rowend,dropdown - for end of row alien drop down & reverse dir.
;  Works ok, aliens travel across/down in desired way.
;-> FREAX5.S (11.7.14)
;  Imp BaseMove, BaseDraw - Move/Draw player base
;  Imp KeyHit for player base control
;-> FREAX6.S (11.7.14)
;  Start Imp MissileFire, MissileMove..
;  Imp v crude removal of aliens on hit, Level over detect..
;-> FREAX7.S (12.7.14)
;  Imp BombDrop, BombMove - Alien bomb drop & animate..
;-> FREAX8.S (13.7.14)
;  Tidy odd bits (Copy 64 chrs, Use SHIFT for fire, etc..)
;  Imp AlienHit/AlienDelete - detect/find alien hit, ripple delete from array
;-> FREAX9.S (14.7.14)
;  2do - Levels, Missile align, score, titlesc, alien bmp correct, Alt dirs, bonus pills. Fast slow bombs/aliens, double bullets..
;  Imp Title screen. Imp score,hiscore. Imp levels.
;  Tidy up more code. Imp ALIENGAPX - 9 pos gap helps animation.
;-> FREAX10.S (14.7.14)
;  Imp SLOWBOMBS, ALT_DIR..
;-> FREAX11.S (15.7.14)
;-> FREAX12.S (16.7.14)
;   2do: Color. Accurate Collide/Drop. Onscreen Score (hex2dec?). +4/128. Shelters. Better rnd. Difficulty opt. Multi lifes.
;  Imp C16/+4 version. Alter rand function.
;-> FREAX14.S (17.7.14)
;  Imp test for alien reraching bottom
;  Imp numbers
;-> FREAX15.S (18.7.14)
;  Imp end game anim
;  HiScore fix
;-> FREAX16.S (20.7.14)
;  Imp on screen score during play.
;-> FREAX17.S (21.7.14)
;  Imp nlifes - multiple bases
;-> FREAX18.S (21.7.14)
;  Imp +4 score
;-> FREAX19.S (25.7.14)
;  Imp Level display
;-> FREAX20.S (26.7.14)
;  2do: mode (unidir, speed etc), move kill alien to end of scan, imp up/down scan. Ignore alien chr on drop. Missile shave alien - do not erase.
;  Imp Shelter. Imp Exit key.
;-> FREAX21.S (1.8.14)
;  Cont Imp Shelters.
;-> FREAX22.S (2.8.14)
;  Cont shelters..
;-> FREAX23.S (3.8.14)
;  2do: simple color? sound?
;-> FREAX24.S (4.8.14)
;  Imp simple fixed color. (USE_COLOUR)
;-> FREAX25.S (2.4.15)
;  Small fixes..
;-> FREAX27.S (6.3.24)
;   Imp FAST_ALIENS
;   Increase ALIENGAPX from 9 to 10.
;   Display level 10+ properly, put invaders to top after each 8 levels. Speedup after level 8.
;   imp TableAlienDir for alien directions. 
;-> FREAX28.S (14.3.24)
;   Inc NLIFES to 5. 
;   Imp MODE flag (bit 0=speed, bit 1=normal game)
;-> FREAX29.S (15.3.24)
; Small changes/bugs: improve Zero Page usage (Inp ZPTEMP2), move bomb table to tape buff etc.
;-> FREAX30.S (17.3.24)
; Imp USE_SOUND and SoundPlay routines. Platform specific translations to real sound registers are done for each platform.
;-> FREAX31.S (18.3.24)
; Tweak sound routines.
; Imp DrawIfNotAlien PATCH Replaces "sta (palien),y" stops glitches where bomb overdraws alien, check to see first..
;-> FREAX32.S (19.3.24)
;
; 2do: Anim on loss. Redraw missile properly. VIC28 col. Sound. Some Faster rows/etc. 
; 2do: Scan alien array in 2 directions, service 1 direction each time. Bonus life every 512,1024,2048 (like bitcoin).. 
; 2do: Imp separate speed (0-3),mode.. Joystick. Better C64 sound. Replace ROM PRINTINT with custom routine, free up zeropage.
; 2do: Better GFX/SFX for alien/base hit, next level etc. Better title screen. Special mothership etc. C128/oric version?
;--------------------------------------------------------------------------

.define IS_C64 1
.define IS_V20 0
.define IS_C16 0
; .define IS_C128 0
; .define IS_PET 0

.define EXTRA       1		; Advanced features
.define SLOWBOMBS   1		; Set if some bombs are slow
.define ALT_DIR     1		; Set for alternating alien direction on each row
.define SHELTERS    1
.define USE_COLOUR  1
.define USE_SOUND   1
.define FAST_ALIENS 0
.define TEST_CODE   0		; Additional testing options..
.define DEBUG       0

.if IS_C64			; CBM 64..
  PRGSTART = $0801
  ZTIME  = $A2		; TIME var
  ZPTEMP = $A3		; 32 Bytes zero page storage to be reused..  (try $A3,2D) 
  ZPTEMP2 = $57		; Another zero page temp work area (FAC1,2) not cleared on running, can be overwritten by RPRINTINT..
  VDUP = $0400		; VDU
  VDUX = 40
  VDUY = 25
  COLORP  = $D800
  COLTOP  = 4
  COLBASE = 1
  COLLOWER = 5
  COLSHELTER = 3
  TapeBuf = $033C		; 192 bytes storage..
  ChrSet  = $3000
  ZKEYDOWN = $C5	; Key down (peek(197) )
  ZKEYSHIFT = $028D	; Shift down
  KEYLEFT  = 33
  KEYRIGHT = 41
  KEYFIRE  = 62
  KEYQUIT  = 23
  
  KEYTEST  = 56		; TEST_CODE cheat/test keys: 1=56,2=59,3=8..+=40,-=43
  KEYLIFE  = 59
  
  VICreg1 = $d011	; VIC Control Register 1 (bit 7= raster bit 8)
  VICmem  = $d018	; VIC Memory Control Register: Bit:4-7 (VDU Base Bit 13-10) Bit:1-3 (Char Set Bit 13-11)
  VICbord = $d020	; VIC border colour
  RPRINTINT = $BDCD        ; PRINT +VE INT (A;X) (C64=BDCD)
  RPRINTBUFF = $0100	; Buffer for printing an int..
  PLAYSPEED = 2
  TOPPOSY = 2
  NALIENY = 6			; #rows
  NALIENX = 1+VDUX/4		; #aliens across

.endif


.if IS_V20			; VIC20 (unexpanded)
  PRGSTART = $1001
  ZTIME  = $A2		; TIME var
  ZPTEMP = $A3		; 32 Bytes zero page storage to be reused..  (try $A3,2D) 
  ZPTEMP2 = $57		; Another zero page temp work area (FAC1,2) not cleared on running, can be overwritten by RPRINTINT..
  VDUP = $1E00		; VDU
  VDUX = 22
  VDUY = 23
  COLORP = $9600
  COLTOP  = 4
  COLBASE = 1
  COLLOWER = 5
  COLSHELTER = 3
  TapeBuf = $033C
  ChrSet = $1c00
  ZKEYDOWN = $C5	; Key down
  ZKEYSHIFT = $028D	; Shift down
  KEYLEFT  = 12
  KEYRIGHT = 13
  KEYFIRE  = 48
  KEYQUIT  = 26

  KEYTEST  = 0		; TEST_CODE cheat/test keys: 1=0,2=56,3=1..+=5,-=61
  KEYLIFE  = 56

  VICraster = $9004	; Cur Raster/2
  VICbase = $9005	; Scr/Chr base.
  VICcol = $900f	; Scr/Border colour
  RPRINTINT = $DDCD        ; PRINT +VE INT (A;X) 
  RPRINTBUFF = $0100	; Buffer for printing an int..
  PLAYSPEED = 1
  TOPPOSY = 2			; VIC20 start high
  NALIENY = 5			; #rows
  NALIENX = 1+VDUX/4		; #aliens across
.endif

.if IS_C16			; CBM16/+4..
  PRGSTART = $1001
  ZPTEMP = $D0		; 32 Bytes reusable zero page storage..  (try $DO,2D) 
  ZPTEMP2 = $57		; Another zero page temp work area (FAC1,2) not cleared on running, can be overwritten by RPRINTINT..
  VDUP = $0C00		; VDU start
  VDUX = 40
  VDUY = 25
  COLORP = $0800
  COLTOP  = 68
  COLBASE = 126
  COLLOWER = 69
  COLSHELTER = 99
  TapeBuf = $0332
  ChrSet  = $2000
  ZKEYDOWN = $07F6	; (2038) Key down
  ZKEYSHIFT = $0543	; Shift down

  KEYLEFT  = 33
  KEYRIGHT = 41
  KEYFIRE  = 62
  KEYQUIT  = 23
  
  KEYTEST  = 56		; TEST_CODE cheat/test keys: 1=56,2=59,3=8..+=54,-=46
  KEYLIFE  = 59

  TEDmemctl  = $FF12	; Bitmap/Character Memory Control (Bit0-1:Sound MSB, Bit 2:chr/bmp ROM/RAM, Bit3-5:Bmp Base 8K steps)
  TEDchrbase = $FF13	; System Clock and Character Memory Control (Bit 2-7:chr base 1K steps)
  TEDvduloc  = $FF14	; Video ram loc 2k steps (Bit 3-7)
  TEDvducol  = $FF15	; Screen background colour (Bit 0-3:Color, Bit 4-6:Luminance)
  TEDbordcol = $FF19	; Border colour
  TEDrasthi  = $FF1C	; Raster line hi bit (Bit 0)
  TEDraster  = $FF1D	; Current raster line (Bit 0-7)
  
  TEDmemctl_TEXT = 196	; Bit 2 on for Text (use ROM)
  TEDmemctl_GFX  = 192	; Bit 2 off for GFX (use RAM)

  RPRINTINT = $A45F	; PRINT +VE INT (A;X) (C64=BDCD)
  RPRINTBUFF = $0100	; Buffer for printing an int..
  PLAYSPEED = 2
  TOPPOSY = 2	
  NALIENY = 6			; #rows
  NALIENX = 1+VDUX/4		; #aliens across

.endif

  ;RPRINTINT = $8E32        ; C128:PRINT +VE INT (A;X) (C64=BDCD)

KCHROUT = $FFD2		; Write char (A)
KGETIN  = $FFE4		; Read chr (A)

.IF (USE_CA65 = 1)			; Kludge to add start address for CA65 asm for .prg files
 .ORG PRGSTART-2
 .WORD PRGSTART
.ELSE
 .ORG PRGSTART
.ENDIF

;--------------------------------------------------------------------------
;    Allocate some program vars to zero page.
;--------------------------------------------------------------------------

naliens  = ZPTEMP		; no of aliens
calien   = ZPTEMP+1		; Current alien # (loops until all aliens moved, then back to zero)
palien	 = ZPTEMP+2		; Ptr to cur alien VDU pos. (Also used as gen purpose ZP pointer)
ycount   = ZPTEMP+4
rvsdir   = ZPTEMP+5		; Bit 7 set if reverse alien dir (left)
rowend   = ZPTEMP+6		; Set when end of row hit
dropdown = ZPTEMP+7		; Set to drop all aliens down 1 row.
alienx   = ZPTEMP+8		; Alien X pos (in 1/4 chrs, ie: 0..159 on 40 col scrn)
alieny   = ZPTEMP+9		; Alien Y pos (ie: 0..24)
basepos  = ZPTEMP+10	; Pos of player base (1/4 chrs)
basedir  = ZPTEMP+11	; Base dir (0,1,$ff)
missilex = ZPTEMP+12	; Player Missile X pos (1/4 chr)
missiley = ZPTEMP+13	; Missile Y pos
movephase= ZPTEMP+14	; Inc each service loop (1/50 sec), used for timing, etc.
nbombs   = ZPTEMP+15	; # bombs in flight
basehit	 = ZPTEMP+16	; Set if base hit by bomb (1) or aliens reach end (FF)
						; Clear upto here for NEWLEVEL
level    = ZPTEMP+17	; Current level
score    = ZPTEMP+18	; Player Score (2 bytes)
						; Clear upto here for NEWGAME, following vars permanent..
nlifes   = ZPTEMP+20	; # lifes left
playmode = ZPTEMP+21	; Misc game control bits. 
hiscore  = ZPTEMP+22    ; game hiscore (2 bytes)
soundfreq= ZPTEMP+24	; Sound: Freq of 2 sound registers, 0=off (2 bytes)
soundlen  = ZPTEMP+26	; Sound Length of note (Tick count)

ZPUSED    = 25		; #Mem to clr at ZPTEMP on startup.
ZPCLRLEV  = 16		; Clear first 16 bytes for new level
ZPNEWGAME = 19		; Clear first 19 bytes for new game

	; Use another zero page temp scratch pad area ZPTEMP2 (FAC1,2), NOT INITIALISED
pscount   = ZPTEMP2		; Temp count Speed..
dirbits   = ZPTEMP2		; Temp direction bits..
cbomb     = ZPTEMP2		; Cur bomb
bombpos   = ZPTEMP2+1	; Bomb phase temp (0-3)
bomby	  = ZPTEMP2+2	; bomb y pos..
randseed  = ZPTEMP2+3	; Rand # seed (2 bytes) (doesnt matter if upset by PRINTINT)
alienwave = ZPTEMP2+5	; Increment every time whole alien wave has been moved..
soundtemp = ZPTEMP2+6	; Used for zero page temp by sound routines..
bombtemp  = ZPTEMP2+6

;--------------------------------------------------------------------------
;  Platform specific executable code here..
;--------------------------------------------------------------------------

.if IS_C64		; CBM64 Specific code

  .BYTE $0B,$08,$66,$19,$9E,$32,$30,$36,$31,0,0,0		; BASIC:  6502 SYS2061

InitCode:
   lda #0
   sta VICbord		; Set screen black
   sta VICbord+1
   jsr MainCode

ModeNormal:		; Normal text mode
   lda #$15
   sta VICmem		; Default VDU=$0400,CHRSET=$D000
   rts

ModeGfx:		; Hi-res graphics mode
   lda #$1d
   sta VICmem		; Set VDU=$0400,CHRSET=$3000
   lda #147
   jmp KCHROUT

SyncFly:		; Sync action to screen refresh
  sei 
SyncF1:
  lda VICreg1
  bmi SyncF1
SyncF2:
  lda VICreg1
  bpl SyncF2
  cli
  rts

Int2Asc:		; Convert +ve int (A;X) to Asc @$0100 (see RPRINTINT)
  sta $62
  stx $63
  ldx #$90
  sec
  jsr $bc49
  jmp $bddf
  
.if USE_SOUND		; Translate generic sound usage (2 osc, 1 byte for freq.) into platform specific output.

C64SID = $D400		; SID chip address (54272)
C64SID_VOL = C64SID + 24

  ; SoundOn: Initialise sound routine
SoundOn:
  jsr SoundOff
  lda #15
  sta C64SID_VOL
  lda #9
  sta C64SID+5		; Voice 1
  sta C64SID+12		; Voice 2
  ;lda #9
  ;sta C64SID+6
  rts
  
  ; Playsound: X=voice (0 or 1) A=Freq, Y=Duration
SoundPlay:
  pha
  sta soundfreq,x
  tya 
  sta soundlen,x
  ldy #0
  txa 
  beq Sound2
  ldy #7
Sound2:
  pla
  lsr
  lsr
  sta C64SID+1,y
  ;lda #0
  ;ror
  ;sta C64SID,y
  lda #16				; Trigger sound..
  sta C64SID+4,y
  lda #17
  sta C64SID+4,y
  rts;

  
  ; SoundTick: Call this every flyback to service sound routines. 
SoundTick:
  ldx #1
SoundTick1:
  lda soundlen,x
  beq SoundTick2		; Voice (x) not playing, do nothing
  dec soundlen,x
  bne SoundTick2		; Voice (x) still playing..
;  txa
;  beq SoundTick3
;  lda #7
;SoundTick3:
;  tay  
;  lda #0
;  sta C64SID+1,y
SoundTick2:
  dex
  bpl SoundTick1
  rts

SoundOff:
  ldx # 24
  lda #0
Sound1:
  sta C64SID,x
  dex
  bpl Sound1
  rts
  
.endif 			; USE_SOUND

.endif			; IS_C64


.if IS_V20		; VIC20 Specific code

  .BYTE $0B,$10,$66,$19,$9E,$34,$31,$30,$39,0,0,0		; BASIC:  6502 SYS4109

InitCode:
   lda #8
   sta VICcol		; Set screen black.
   jsr MainCode

ModeNormal:		; Normal text mode
   lda #$f0
   sta VICbase
   rts

ModeGfx:		; Hi-res graphics mode
   lda #$ff
   sta VICbase
   lda #147
   jmp KCHROUT

SyncFly:		; Sync action to screen refresh
  lda #100
  sei
SyncF1:
  cmp VICraster
  bne SyncF1
  cli
SyncF2:
  ;cmp VICraster
  ;beq SyncF2
  rts

;SyncFly:		; Sync action to screen refresh
;  lda ZTIME
;Sync1:
;  cmp ZTIME
;  beq Sync1
;  rts

Int2Asc:		; Convert +ve int (A;X) to Asc @$0100 (see RPRINTINT)
  sta $62
  stx $63
  ldx #$90
  sec
  jsr $dc49
  jmp $dddf

.if USE_SOUND		; Translate generic sound usage (2 osc, 1 byte for freq.) into platform specific output.

V20_VOX = $900A		; VIC sound address (Bit 7 enables) (registers: voxLo, voxMid, voxHi. noise, vol)
V20_VOL = $900E

  ; SoundOn: Initialise sound routine
SoundOn:
  jsr SoundOff
  lda #15
  sta V20_VOL
  rts
  
  ; Playsound: X=voice (0 or 1) A=Freq, Y=Duration
SoundPlay:
  pha
  tya 
  sta soundlen,x
  pla
SoundSetFreq:			; Just set voice freq.
  sta soundfreq,x
  sec
  ror					; bit 7 enables voice
  sta V20_VOX+1,x
  rts

  
  ; SoundTick: Call this every flyback to service sound routines. 
SoundTick:
  ldx #1
SoundTick1:
  lda soundlen,x
  beq SoundTick2		; Voice (x) not playing, do nothing
  dec soundlen,x
  bne SoundTick2		; Voice (x) still playing..
  lda #0
  sta V20_VOX+1,x		; Stop voice (x)
SoundTick2:
  dex
  bpl SoundTick1
  rts

SoundOff:		; Clear sound registers, kill any sound still playing.
  ldx # 4
  lda #0
  sta soundlen
  sta soundlen+1
Sound1:
  sta V20_VOX,x
  dex
  bpl Sound1
  rts
  
.endif 			; USE_SOUND

.endif			; IS_V20


.if IS_C16		; C16 Specific code

  .BYTE $0B,$10,$66,$19,$9E,$34,$31,$30,$39,0,0,0		; BASIC:  6502 SYS4109

InitCode:
   lda #0
   sta TEDvducol	; Set screen black
   sta TEDbordcol	; Set screen black
   jsr MainCode

ModeNormal:		; Normal text mode
   lda #208
   sta TEDchrbase
   lda #TEDmemctl_TEXT		; 196
   sta TEDmemctl			; $FF12
   rts

ModeGfx:		; Hi-res graphics mode
   lda #32				; Set chrset base to $2000
   sta TEDchrbase		
   lda #TEDmemctl_GFX	; Set chrset to RAM, bit 2 off (192)
   sta TEDmemctl		; TED $FF12
   lda #147
   jmp KCHROUT

SyncFly:		; Sync action to screen refresh
   lda #200
   sei
SyncF1:
   cmp TEDraster
   bne SyncF1
   cli
   rts

Int2Asc:		; Convert +ve int (A;X) to Asc @$0100 (see RPRINTINT)
  sta $62
  stx $63
  ldx #$90
  sec
  jsr $a2ce
  jmp $a471

.if USE_SOUND		; Translate generic sound usage (2 osc, 1 byte for freq.) into platform specific output.

C16_TED = $FF00
C16_VOLUME = $FF11		; TED volume (bits 0-3) bit4=Enable voice 1, bit5=Enable voice 2 
C16_VOX1LO = $FF0E		; Voice 1 lo bits
C16_VOX1HI = $FF12		; Voice 1 hi bits (bits 0,1) OTHER BITS leave alone!
C16_VOX2LO = $FF0F		; Voice 2 lo bits
C16_VOX2HI = $FF10		; Voice 2 hi bits (bits 0,1) rest unused.

  ; SoundOn: Initialise sound routine
SoundOn:
  jsr SoundOff
  lda #15		; Enable sound..
  sta C16_VOLUME
  rts

C16_VoiceMask:		; Lookup to set/clear voice1/2 bits in C16_VOLUME ($FF11)
 .BYTE $EF,$DF
C16_VoiceBit:  
 .BYTE $10,$20
C16_VoiceIndex:		; Index for location of high bits of TED sound
.BYTE $12,$10		; TED+.. (Voice1=$FF12, Voice2=$FF10)
 
  ; Playsound: X=voice (0 or 1) A=Freq, Y=Duration
SoundPlay:
  pha
  tya 
  sta soundlen,x
  lda C16_VoiceBit,x	; Enable voice (x)
  ora C16_VOLUME
  sta C16_VOLUME
  lda # (TEDmemctl_GFX / 4)		; init temp used for calc of sound high bits 
  sta soundtemp
  pla
SoundSetFreq:			; Just set voice freq.
  sta soundfreq,x
  asl
  rol soundtemp
  asl
  rol soundtemp
  sta C16_VOX1LO,x
  lda C16_VoiceIndex,x		; Find place to save sound hi bits 
  tay
  lda soundtemp
  sta C16_TED,y		; Save hi sound bits in TED+.. (Voice1=$FF12, Voice2=$FF10) 
  rts

  ; SoundTick: Call this every flyback to service sound routines. 
SoundTick:
  ldx #1
SoundTick1:
  lda soundlen,x
  beq SoundTick2		; Voice (x) not playing, do nothing
  dec soundlen,x
  bne SoundTick2		; Voice (x) still playing..
  lda C16_VoiceMask,x	; Disable voice (x)
  and C16_VOLUME
  sta C16_VOLUME	
SoundTick2:
  dex
  bpl SoundTick1
  rts

SoundOff:		; Clear sound registers, kill any sound still playing.
  lda #0
  sta soundlen
  sta soundlen+1
  sta C16_VOLUME
  sta C16_VOX2HI
  lda C16_VOX1HI
  and # $fc
  sta C16_VOX1HI
  rts

.endif 			; USE_SOUND

.endif			; IS_C16

.if 0			; TEST - Set to 1 to waste a blob of memory.. (512 bytes)

.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.BYTE 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

.endif

;--------------------------------------------------------------------------
;  Freax - alien specific code
;--------------------------------------------------------------------------


MAXALIEN = (NALIENX * NALIENY)
MAXBOMBS = 10			; Max# bombs in flight

TOPPOSX = (VDUX*2) - (NALIENX * 4) - (NALIENX/2)	; Start pos for top left alien (1/4 chrs)
LEVELMASK = 7			; Used to calc lowest Y start position for aliens

ALIENGAPX = 10			; X-Gap between aliens
ALIENGAPY = 2			; Y-Gap between rows of aliens

XPOSMIN = 3			; Min/Max positions (reverse motion at these points)
XPOSMAX = (VDUX*4) - 7

YPOSMASK = 31		; Mask out hit bits alien Y pos (hi bits used to indicate alien dir, speed, etc)
ALIEN_RVSBIT  = $80		; Set in alieny position to indicate alien row reverse direction
ALIEN_FASTBIT = $40		; Set in alieny position to indicate fast alien row

PLAYMODE_MASK = 3
PLAYMODE_FAST = 1		; playmode fast bit
PLAYMODE_NORM = 2		; playmode normal alien bit

SHELTERWIDTH = 3 + (VDUX/40)
BASESTARTPOS = VDUX*2			; Base start pos in 1/4 chrs
BASEVDU = VDUP + VDUX * (VDUY - 1)	; Base VDU line loc
BASEMAX = (VDUX*4) - 8
NLIFES  = 5			; Default # lifes

CHRSHELTER = 1		; Shelter
CHRALIEN   = 4		; Alien chr start
CHRBASE    = 14		; Player Base chrs..
CHRMISSILE = 23		; Player Missile chrs
CHRBOMB    = 27		; Alien Bomb chrs
CHRNUM     = 33		; Number chrs.. (0..9)
CHRBANG    = 43


	; Assorted tables..
	
tAlienX = TapeBuf				; Table store for alien X/Y loc. (1/4 chr, ie:[0-159,0-24] for 40x25 VDU)
tAlienY = tAlienX + MAXALIEN

tBombX    = TapeBuf + MAXALIEN*2		; Table of bomb positions, tBombX/Y[MAXBOMBS] (was FreeMem + VDUY*2)
tBombY    = tBombX + MAXBOMBS + 1

tVduPosLo = FreeMem	 			; Table with start VDU pos for each screen line
tVduPosHi = FreeMem + VDUY	

;--------------------------------------------------------------------------
;  Assort general i/o routines
;--------------------------------------------------------------------------

ioRand:			; Simple psuedo-random function, return # in (Acc)
  inc randseed+1
  lda randseed
  asl
  rol
  rol
  adc randseed+1
  tay
  eor ChrData,y
  ;eor ZTIME
  sta randseed
  rts

;--------------------------------------------------------------------------
;  NewGame - Start a new game level
;--------------------------------------------------------------------------

TableAlienDir:		; Table indicates directions of alien rows for levels (wraps after 8)
  .BYTE %10101010, %00000000, %11001100, %11100000, %10101010, %11001000, %01000000, %00000000

ClrMem:
  ldx #ZPUSED
ClrMemX:
  lda #0		; Clr Zero page vars
  sta ZPTEMP,x
  dex
  bne ClrMemX
  rts

NewGame:		; Start new game, first level..
  lda #NLIFES
  sta nlifes
  ldx #ZPNEWGAME
  .byte $2c

NewLevel:		; Start next level..
  ldx #ZPCLRLEV
  jsr ClrMemX
  
  .if USE_COLOUR	; Set color for some lines..
    ldx #VDUX*2-1
NewCol:
    lda #COLTOP
    sta COLORP,x
    lda #COLBASE
    sta COLORP-VDUP+BASEVDU-VDUX,x
    lda #COLSHELTER
    sta COLORP-VDUP+BASEVDU-VDUX*5,x
    sta COLORP-VDUP+BASEVDU-VDUX*3,x
    lda #COLLOWER
    sta COLORP-VDUP+BASEVDU-VDUX*9,x
    sta COLORP-VDUP+BASEVDU-VDUX*7,x
    dex
    bpl NewCol
  .endif

  ldx #>VDUP		; Clear VDU
  stx palien+1
  ldx #>(VDUX*VDUY)
  lda #0
  tay 
New2:
  sta (palien),y
  iny
  bne New2
  inc palien+1
  dex
  bpl New2 

  .if SHELTERS
    ldx #0
NewShelter2:
    ldy #SHELTERWIDTH
NewShelter:
    lda #CHRSHELTER
    sta BASEVDU-VDUX*4+VDUX/4-3,x
    sta BASEVDU-VDUX*4+VDUX/2-2,x
    sta BASEVDU-VDUX*4+(3*VDUX)/4,x
    inx
    dey
    bne NewShelter
    txa
    clc
    adc #VDUX-SHELTERWIDTH
    tax
    cpx #VDUX*3
    bcc NewShelter2
  .endif

  lda #BASESTARTPOS	; Player base start pos
  sta basepos
  lda #MAXALIEN
  sta naliens		; Calc no of aliens
  lda #NALIENY
  sta ycount
  lda level			; Use level to derive top of alien start pos.
  and # LEVELMASK   ; Aliens go back to top after level 8..
  tay
  lda TableAlienDir,y
  sta dirbits 		; Bits indicate direction of each row of aliens (hi..lo)
  lda playmode			; Has user set "normal" alien game?
  and # PLAYMODE_NORM
  beq NewNext4
  lda #0
  sta dirbits
 NewNext4: 
  tya
  clc 
  adc #TOPPOSY
  sta alieny		; Save alien top y position.
  
  ldy #0
NewNextY:
  ldx #NALIENX
  lda #TOPPOSX
  sta alienx		; Init Alien top X/Y start pos 
NewNextX:
  lda alienx
  sta tAlienX,y
  clc
  adc # ALIENGAPX
  sta alienx
  lda alieny
  .if ALT_DIR
   bit dirbits		; Test bit 7 for alien direction
   bpl NewNext3
   lda # ALIEN_RVSBIT
NewNext3:
   ora alieny		; Set hi bit to reverse dir..
  .endif
  sta tAlienY,y
  iny
  dex
  bne NewNextX
  inc alieny  
  .if (ALIENGAPY > 1)
   inc alieny  
  .endif
  asl dirbits		; Next direction into bit 7
  dec ycount
  bne NewNextY

  jsr OutStatus		; Display score

CalcVduPos:		; Calc loopup table with VDU pos start each line (0..VDUY-1)
  lda #<VDUP
  ldy #>VDUP
  ldx #0
CalcLoop:
  sta tVduPosLo,x
  pha
  tya
  sta tVduPosHi,x
  pla
  clc
  adc #VDUX
  bcc Calc1
  iny
Calc1: 
  inx
  cpx #VDUY
  bne CalcLoop
  rts

GetAlienX:		; Read info for current alien (x)
  lda tAlienY,x
  sta alieny
  and #YPOSMASK		; Isolate Y pos bits from other flags
  tay
  lda tAlienX,x
  sta alienx

CalcPosFromXY:		; Calc real VDU loc (palien) from 1/4 chr (x,y) (x = Acc, y = Y reg) (reg X unused)
  lsr
  lsr
  clc
  adc tVduPosLo,y
  sta palien		; Calc Real VDU pos from Alien X/Y pos
  lda tVduPosHi,y
  adc #0
  sta palien+1
  rts

SetAlienX:		; Save info for current alien (x)
  lda alienx
  sta tAlienX,x
  lda alieny
  sta tAlienY,x
  rts

;--------------------------------------------------------------------------
;  DrawAlien - Draw alien 
;--------------------------------------------------------------------------

DrawAlien:		; Draw single alien (x)
  jsr GetAlienX
  lda alienx
  and #3		; Calc alien phase 0-3
  asl
  adc #CHRALIEN
DrawAlienAcc:		; Fill alien with chr (a)
  ldy #0
  sta (palien),y
  iny
  adc #1	
  sta (palien),y
  cmp #CHRALIEN+6
  bcc DrawAlienRet
  adc #0	
  iny
  sta (palien),y
DrawAlienRet:
  rts

WipeAlien:
  jsr GetAlienX
  ldy #0
  tya
  sta (palien),y
  iny
  sta (palien),y
  lda alienx
  and #3		; Calc alien phase 0-3
  cmp #3
  bcc WipeAlienRet
  lda #0
  iny
  sta (palien),y
WipeAlienRet:
  rts
  

DrawScreen:		; Draw alien array
  ldx naliens
  dex
Draw1:
  jsr DrawAlien
  dex
  bpl Draw1
  rts

;--------------------------------------------------------------------------
;  AlienNext - move current alien, advance to next on list
;--------------------------------------------------------------------------

AlienNext:		; Animate next alien
  lda naliens
  beq AlienRet
  ldx calien
  cpx naliens
  bcc Alien2
  
  inc alienwave		; Increments every time alien wave moves..
  ldx #0		; End of updating array of aliens, back to top alien.
  stx dropdown
  lda rowend		; Reverse dir?
  beq Alien2
  stx rowend
  lda rvsdir
  eor # ALIEN_RVSBIT 
  sta rvsdir		; Flip hi bit (rvsdir) 
  dec dropdown		; Drop down all aliens next iteration
Alien2:
  stx calien

  jsr WipeAlien		; Clr old Alien
  lda dropdown		; Drop-down flag set?
  beq AlienMove
  inc alieny		; Move all aliens down 1 row
  lda alieny
  and #YPOSMASK
  cmp #VDUY-2
  bcc AlienRedraw
  dec basehit		; Alien reaches bottom, game over
  bne AlienRedraw

AlienMove:
;  .if FAST_ALIENS
;    lda alieny
;    and # ALIEN_FASTBIT
;    beq AlienMove4
;    lda movephase
;    and #1
;    beq AlienMove4
;    rts
;AlienMove4:  
;  .endif
  lda alieny
  eor rvsdir
  bmi AlienBackDir	; Alien moves Dir back..
  .if FAST_ALIENS
    and # ALIEN_FASTBIT
    beq AlienMove2
    inc alienx		; Double movement for fast aliens
AlienMove2:  
  .endif
  inc alienx
  lda alienx
  cmp #XPOSMAX		; reached right hand of VDU?
  bcc AlienRedraw
  bcs AlienRvs		; Reverse dir at end of block

AlienBackDir:		; Decrement alien phase/pos
  .if FAST_ALIENS
    and # ALIEN_FASTBIT
    beq AlienMove3
    dec alienx		; Double movement for fast aliens
AlienMove3:  
  .endif
  dec alienx
  lda alienx
  cmp #XPOSMIN
  bcs AlienRedraw

AlienRvs:		; Reverse dir at end of block
  inc rowend

AlienRedraw:
  jsr SetAlienX		; Save updated (x,y) pos for alien (x)
  jsr DrawAlien		; Draw alien in new pos.
  jsr BombDrop
Alien3:
  inc calien		; next alien
AlienRet:
  rts


AlienHit:		; Test for alien hit
  ldx naliens		; Search Alien table for match to (missilex,missiley)
AlienHitNext:
  dex
  bmi AlienNotFound
  lda tAlienY,x
  and #YPOSMASK		; Mask off hi bits
  cmp missiley
  bne AlienHitNext	; Vertical pos not matching, next..
  lda missilex
  sec
  sbc tAlienX,x
  cmp #12		; Width of alien in pixels - "hit" area.
  bcc AlienFound	; Alien (x) hit found..
  bcs AlienHitNext
AlienNotFound:
  clc			; Carry clear -> not found
  rts  

AlienFound:		; Alien hit found, 
  jsr WipeAlien		; Clr Alien (x)

AlienDelete:		; Ripple delete Alien (x)
  cpx calien
  bcs AlienDel1
  dec calien		; Adjust (calien) in main wave, to keep animation consistant
AlienDel1:
  inx
  lda tAlienY,x
  dex
  sta tAlienY,x
  inx 
  lda tAlienX,x
  dex
  sta tAlienX,x
  inx
  cpx naliens
  bcc AlienDel1
  dec naliens
  lda level
  lsr
  sec
  adc score		; Increment score by (1+(level/2))
  sta score
  bcc AlienDel2
  inc score+1
AlienDel2:		; Drop through..

OutStatus:		; Update status line
  ldx #CHRALIEN
  stx VDUP+VDUX-8	; Display level alien gfx
  inx
  stx VDUP+VDUX-7
  lda #<(VDUP+VDUX-6)
  sta palien
  lda #>(VDUP+VDUX-6)
  sta palien+1
  ldx level
  inx 
  lda #0
  jsr OutputInt
  
  ldx #CHRBASE
  stx VDUP+VDUX-3	; Display base gfx
  inx
  stx VDUP+VDUX-2
  lda nlifes
  clc
  adc #CHRNUM
  sta VDUP+VDUX-1	; Display no of lifes left
OutScore:		; Update score
  lda #<VDUP
  sta palien
  lda #>VDUP
  sta palien+1
  ldx score
  lda score+1
				; Drop through..
OutputInt: 		; Output 16 bit INT in (A;X), poke to screen at (palien)
  jsr Int2Asc	; Convert int in (A;X) to ascii at (RPRINTBUFF) ($0100)
  ldy #0
OutS1:
  lda RPRINTBUFF,y
  beq OutS2
  clc 
  adc # (CHRNUM + 256 - 48)  ; Correct to chrset
  sta (palien),y
  iny
  bne OutS1
OutS2:
  ;; sec			; Carry set if alien found/destroyed
  rts

;--------------------------------------------------------------------------
;  Bomb..  - Bomb drop/move routines
;--------------------------------------------------------------------------

BombDrop:		; Drop bomb from (alienx,alieny)
  jsr ioRand
  lda level		; If (level+3 > (rand() & 255)) drop a bomb..
  clc
  adc #3
  cmp randseed
  bcc BombExit		; Dont drop..
  ldx nbombs
  cpx #MAXBOMBS-1
  bcs BombExit		; Bomb table full
  ldy #VDUX*2
  lda (palien),y	; See if alien underneath (2 lines down), if so skip..
  ldy #1
  sec
  sbc # CHRALIEN
  cmp # (CHRBASE-CHRALIEN)
  ;bcc BombExit
  bcs BombDr1
  ldy #3		; Is alien chr, skip 3 lines
BombDr1:
  lda alienx
  sta tBombX,x
  tya
  clc
  adc alieny		; Scale y pos..
  asl
  asl
  sta tBombY,x
  inc nbombs
BombExit:
  rts


; Different bomb pos actions..
; pos0: z=peek(+0): poke+0,0:if isterm(z) end: poke +0,b1
; pos1: z=peek(+40):poke+0,0:if isterm(z) end: poke +0,b2:poke +40,b2+1
; pos2: z=peek(+40):poke+0,0:poke+40,0:if isterm(z) end: poke +0,b3:poke +40,b3+1
; pos3: z=peek(+40):poke+0,0:poke+40,0:if isterm(z) end: poke +40,b0

BombPhase:	; Lookup encodes bomb ypos phase (0-3) to drawn chars
  .byte CHRBOMB+1, CHRBOMB+2, CHRBOMB+4, 0
BombPhase2:
  .byte    0	 , CHRBOMB+3, CHRBOMB+5, CHRBOMB

BombMove:		; Move all bombs down 1 position
  ldx #0		; current bomb
BombLoop:
  cpx nbombs
  bcs BombExit	; All done..
  stx cbomb
  lda tBombY,x
  sta bomby		; temp for later
  lsr
  lsr
  tay
  .if SLOWBOMBS
   lda tBombX,x		; If X pos is odd, its a slow drop bomb
   and #1
   and movephase	; skip on alt cycles
   bne BombNext
  .endif
  lda tBombX,x
  jsr CalcPosFromXY	; Calc VDU pos (palien) from X,Y coords
  ldy #0
  lda bomby
  and #3			; Calc bomb Y phase (0-3)
  sta bombpos
  beq Bomb1	
  ldy #VDUX
Bomb1:
  lda (palien),y	; Save peek of dest hit by bomb
  pha
  ldy #0
  tya
  ; sta (palien),y
  jsr DrawIfNotAlien   ; sta (palien),y  ; Patch - only draw if not overwriting alien
  lda bombpos
  cmp #2
  bcc Bomb2
  tya
  ldy #VDUX
  ; sta (palien),y
  jsr DrawIfNotAlien   ; sta (palien),y  ; Patch - only draw if not overwriting alien
Bomb2:
  pla			; retrieve dest hit by bomb
  beq BombDraw		; empty, cont..
  ldy bomby
  cpy #(VDUY*4)-4
  bcs BombEnd
  cmp #CHRMISSILE
  bcs BombDraw
  cmp #CHRBASE
  bcs BombHit	; Bomb hits player, loose a life..

  cmp #CHRSHELTER
  beq Bomb5
  lda #0		; Shelter hit, erase a chr
  .byte $2c
Bomb5:
  lda #CHRSHELTER+1
  ldy #VDUX
  ; sta (palien),y
  jsr DrawIfNotAlien   ; sta (palien),y  ; Patch - only draw if not overwriting alien
  
BombEnd:		; Ripple-delete cbomb from list
  ldx cbomb
BombEndLoop:
  inx
  lda tBombX,x
  dex
  sta tBombX,x
  inx
  lda tBombY,x
  dex
  sta tBombY,x
  inx
  cpx nbombs
  bcc BombEndLoop
  dec nbombs		; nbombs--;
  bne BombNext
  rts

BombDraw:		; Draw bomb 
  ldx bombpos
  lda BombPhase,x	; Use lookup table to convert bomb y phase (0-3) to chars..
  beq Bomb3
  ldy #0
  ; sta (palien),y
  jsr DrawIfNotAlien   ; sta (palien),y  ; Patch - only draw if not overwriting alien
Bomb3:
  lda BombPhase2,x
  beq Bomb4
  ldy #VDUX
  ; sta (palien),y
  jsr DrawIfNotAlien   ; sta (palien),y  ; Patch - only draw if not overwriting alien
Bomb4:
  ldx cbomb
  inc tBombY,x		; Move bomb down 1/4 chr
BombNext:
  inc cbomb
  ldx cbomb
  jmp BombLoop

BombHit:		; Bomb hits base
  inc basehit
  rts

DrawIfNotAlien:		; PATCH Replaces sta (palien),y   stops "glitches" where bomb overdraws alien, check to see first..
  sta bombtemp
  lda (palien),y
  cmp #CHRALIEN		; if (peek(bombpos) >= CHRALIEN && peek(bombpos) < CHRBASE) dont draw..
  bcc BombNot1
  cmp #CHRBASE
  bcc BombNot2		; Bomb hit an Alien, dont draw..
BombNot1:
  lda bombtemp
  sta (palien),y
BombNot2:
  rts

;--------------------------------------------------------------------------
;  Missile..  - Missile fire/move routines
;--------------------------------------------------------------------------

MissileFire:		; Try to fire player missile
  lda missilex
  bne MissileRet		; Missile already in flight
  ldx basepos
  inx
  inx
  stx missilex
  lda #VDUY-1
  sta missiley

MissileRet:
  rts

MissileMove:		; Move player missile
  lda missilex
  beq MissileRet
  ldy missiley 
  jsr CalcPosFromXY	; Calc VDU loc
  ldy #0
  tya
  ; sta (palien),y	; Clr old missile
  jsr DrawIfNotAlien		; PATCH Replaces sta (palien),y   stops "glitches" where bomb overdraws alien, check to see first..
  dec missiley
  beq MissileEnd	; Off screen, kill missile.
  lda missilex
  ldy missiley 
  jsr CalcPosFromXY
  ldy #0
  lda (palien),y
  bne MissileHit

  lda missilex		
  and #3
  clc
  adc #CHRMISSILE
  sta (palien),y	; Save updated missile
  rts

MissileHit:		; Missile has hit something
  cmp #CHRALIEN
  bcc MissileShelter
  cmp #CHRBASE
  bcs MissileRet	; Ignore bombs..
  jsr AlienHit		; Test for alien hit
  bcc MissileRet	; Not a hit, continue..
MissileEnd:		; Terminate missile
  ldx #0
  stx missilex
  rts

MissileShelter:		; Missile hits shelter
  tya
  sta (palien),y	; clr dest square
  beq MissileEnd


;--------------------------------------------------------------------------
;  BaseDraw - Service player base drawing
;--------------------------------------------------------------------------

BaseMove:
  lda basedir
  beq BaseDraw		; No movement? just redraw
  jsr BaseClr		; Clear current base on VDU
  lda basepos
  clc
  adc basedir
  cmp #2		; See if base is moving out of range..
  bcc BaseDraw
  cmp #BASEMAX
  bcs BaseDraw
  sta basepos		; Update base pos

BaseDraw:		; Draw player base
  lda basepos		; Convert 1/4 chr pos to chr index (y)
  lsr
  lsr
  tay
  lda basepos
  and #3
  asl 
  adc # CHRBASE
  sta BASEVDU,y
  iny
  adc #1
  sta BASEVDU,y
  cmp # CHRBASE+7
  bcc BaseExit
  iny
  adc #0
  sta BASEVDU,y
BaseExit:
  rts

BaseClr:		; Clr player base
  lda basepos		; Convert 1/4 chr pos to chr index (y)
  lsr
  lsr
  tay
  lda #0
  sta BASEVDU,y
  iny
  sta BASEVDU,y
  iny
  sta BASEVDU,y
  rts

  
CopySet:
  ldx #0		; Copy 64 chrs (512 bytes) of user chr data..
CopyLoop:
  lda ChrData,x
  sta ChrSet,x
  lda ChrData+$0100,x
  sta ChrSet+$0100,x
  ;txa; sta VDUP,x ;sta VDUP+256,x
  dex
  bne CopyLoop
  rts

DumpSet:
  ldx #0		; Copy & Display 256 chrset..
DumpLoop:
  txa
  sta VDUP,x 
  sta VDUP+256,x
  dex
  bne DumpLoop
  rts

KeyHit:
  ldx #0
  lda ZKEYDOWN		; Player key hit..
  cmp #KEYLEFT
  bne Key1 
  dex
Key1:
  cmp #KEYRIGHT
  bne Key2
  inx
  inc randseed		; Help random number generator! User keyhit helps scramble..
Key2:
  stx basedir		; Save ($ff,0,1) to player dir
  cmp #KEYQUIT
  beq KeyQuit		; Exit with carry set (Exit game)
  lda ZKEYSHIFT
  beq Key3
  jsr MissileFire	; Fire a missile
Key3:
  clc
KeyQuit:
  rts

DelaySec:			; Delay for a few secs (life lost, new level, etc)

.if USE_SOUND	
  jsr SoundOn			; Reset Sound (kill any pending)..
.endif

  ldx #100
DelayS1:
  jsr SyncFly
  dex
  bne DelayS1
  
  rts
    
;--------------------------------------------------------------------------
; Service sound for Main Loop
;--------------------------------------------------------------------------

.if USE_SOUND
MainSoundService:
  jsr SoundTick			; Service sound every game loop..
				
  lda alienwave			; Make little pulse sound in time with alien wave moves..
  bmi AlienPulseSkip
  lda #$fc
  sta alienwave
  jsr ioRand
  and #127
  ldx #0
  ldy #1
  jsr SoundPlay
AlienPulseSkip:  

		; Make missile firing sound..
  lda missilex
  beq sndskip		; No Missile
  lda missiley
  lsr
  bcc sndskip
  asl
  asl
  adc #90
  ldx #1			; Voice 1
  ldy #1			; Duration
  jsr SoundPlay
sndskip:
  rts
  
.endif		; USE_SOUND

;--------------------------------------------------------------------------
;  MainCode - Main Loop
;--------------------------------------------------------------------------

.if EXTRA

MainText:
.BYTE 156,147,"  F R E A X",13
.BYTE         " -=-=-=-=-=-",13,13,31
.BYTE "(C) A.MILLETT 2024",13
.BYTE "A FREEWARE GAME. V32",13,13,30
.BYTE "CLEAR THE ALIENS!",13,13
.BYTE " LEFT,RIGHT - I,P",13
.BYTE " FIRE - SHIFT",13
.BYTE " QUIT - X",13
.BYTE 13,13,13,13,13,5,"HIT SPACE TO START."
.BYTE 145,13,145,145,145,145,158,"SCORE:",0
MainText2:
.BYTE " HI:",0
MainText3:
.BYTE 13,13,"MODE (M):",0

PrintStr:	; Print null-term string at (Y;A)
  sta palien
  sty palien+1
PrintS1:
  ldy #0
  lda (palien),y
  beq PrintStrExit
  jsr KCHROUT
  inc palien
  bne PrintS1
  inc palien+1
  bne PrintS1
PrintStrExit:
  rts

.endif

MainCode:			; Main program start, clear memory, initialise char set etc..
  jsr ClrMem
  jsr CopySet		; Init chr set
  
MainNew:
  .if EXTRA
    jsr ModeNormal
    lda #<(MainText)		; Display Title Screen main text
    ldy #>(MainText)
    jsr PrintStr
    
    ldx score				; if (score > hiscore) hiscore = score
    lda score+1
    cmp hiscore+1
    bcc FreaxT2
    bne FreaxT3
    cpx hiscore
    bcc FreaxT2
FreaxT3:
    stx hiscore
    sta hiscore+1
FreaxT2:
    jsr RPRINTINT			; print the score
    
    lda #<(MainText2)		; PRINT "HI:";hiscore
    ldy #>(MainText2)
    jsr PrintStr
    ldx hiscore
    lda hiscore+1
    jsr RPRINTINT
							; PRINT "MODE:";playmode
    lda #<(MainText3)
    ldy #>(MainText3)
    jsr PrintStr
    ldx playmode
    lda #0
    jsr RPRINTINT
    
FreaxT1:					; Ok, wait for user command..
    jsr KGETIN
    ; cmp #88					; X hit, exit program
    ; beq MainExit
    
    cmp #77					; M hit, change playmode..
    bne FreaxT4
    inc playmode			; playmode = (playmode + 1) & PLAYMODE_MASK;
    lda playmode
    and # PLAYMODE_MASK		; Wrap around playmode.
    sta playmode
    bpl MainNew				; Redraw title screen
    
FreaxT4:    
    cmp #32					; SPACE to play..
    bne FreaxT1
  .endif
  
						; Start new game..
  jsr ModeGfx
  jsr NewGame
  
MainLevelLoop:			; Start new level..

.if USE_SOUND			; Enable Sound..
  jsr SoundOn
.endif

  jsr DrawScreen
  jsr BaseDraw
  
MainLoop:  				; Main reloop point for gameplay..

.if USE_SOUND	
  jsr MainSoundService
.endif  
  jsr KeyHit
  bcs MainGameOver
  
.if TEST_CODE		; Assort testing code: Key 1=level++, 2=lifes++
  lda ZKEYDOWN
  cmp #KEYLIFE
  bne TESTCODE2
  inc nlifes
TESTCODE2:  
  cmp #KEYTEST
  bne TESTCODE1
  jmp MainNextLevel
TESTCODE1:
.endif

  jsr MissileMove
  
  lda naliens
  ora nbombs			; Wait till all bombs have fallen..
  beq MainNextLevel		; No more aliens, level Over..
  jsr BombMove
  lda basehit
  bmi MainGameOver
  bne MainBaseHit
  jsr BaseMove		; Move/Redraw player base

		; Animate next alien(s)
  lda playmode
  and # PLAYMODE_FAST  ; Set to 1 if fast
  ldx level
  cpx # 8			; Increase speed after level 8
  adc # PLAYSPEED	; add default speed, and carry set if (level > 8)
  sta pscount
MainSpeedLoop:
  jsr AlienNext  ;  jsr AlienNext
  dec pscount
  bne MainSpeedLoop

  jsr SyncFly
  inc movephase		; Increment move phase
  jmp MainLoop

MainGameOver:
  jsr DelaySec
  jmp MainNew

MainBaseHit:		; Show base destruction 
 dec nlifes
 jsr OutStatus
 jsr BaseClr
 .if EXTRA
MainBH1:
   jsr SyncFly
   jsr BombMove		; Drop remaining bombs..
   inc movephase
   lda nbombs
   bne MainBH1
 .endif
 jsr DelaySec
 lda #0
 sta basehit
 lda nlifes
 bpl MainLoop
 bmi MainGameOver

MainExit:
  rts

MainNextLevel:
  jsr DelaySec
  inc level
  jsr NewLevel
  jmp MainLevelLoop

;--------------------------------------------------------------------------
;  ChrData - User defined chr bitmaps
;--------------------------------------------------------------------------

ChrData:
  .BYTE 0,0,0,0,0,0,0,0
  .BYTE $aa,$55,$aa,$55,$aa,$55,$aa,$55

  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %01000000
  .BYTE %10100010
  .BYTE %01010101
  .BYTE %10101010
  .BYTE %01010101
  ;.BYTE 0,0,0,0,$aa,$55,$aa,$55
  .BYTE 0,0,0,0,0,0,0,$ff

		; Alien phase 0 (CHR 4)
  .BYTE %00010000
  .BYTE %10011111
  .BYTE %10111111
  .BYTE %11101111
  .BYTE %01111001
  .BYTE %00111111
  .BYTE %00100000
  .BYTE %00100000
  
  .BYTE %10000000
  .BYTE %10000000
  .BYTE %11000000
  .BYTE %01100000
  .BYTE %11110000
  .BYTE %11010000
  .BYTE %01010000
  .BYTE %01000000

		; Alien phase 1
  .BYTE %00000100
  .BYTE %00000111
  .BYTE %00001111
  .BYTE %00011011
  .BYTE %00111110
  .BYTE %00101111
  .BYTE %00101000
  .BYTE %00000100
  
  .BYTE %00100000
  .BYTE %11100100
  .BYTE %11110100
  .BYTE %11011100
  .BYTE %01111000
  .BYTE %11110000
  .BYTE %00010000
  .BYTE %00100000

		; Alien phase 2
  .BYTE %00000001
  .BYTE %00001001
  .BYTE %00001011
  .BYTE %00001110
  .BYTE %00000111
  .BYTE %00000011
  .BYTE %00000010
  .BYTE %00000010
  
  .BYTE %00001000
  .BYTE %11111000
  .BYTE %11111100
  .BYTE %11110110
  .BYTE %10011111
  .BYTE %11111101
  .BYTE %00000101
  .BYTE %00000100

		; Alien phase 3 (3 chrs)
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000001
  .BYTE %00000011
  .BYTE %00000010
  .BYTE %00000010
  .BYTE %00000000
  
  .BYTE %01000010
  .BYTE %01111110
  .BYTE %11111111
  .BYTE %10111101
  .BYTE %11100111
  .BYTE %11111111
  .BYTE %10000001
  .BYTE %01000010

  .BYTE %00000000
  .BYTE %01000000
  .BYTE %01000000
  .BYTE %11000000
  .BYTE %10000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
		; Edge with adjacent alien
  .BYTE %00000000
  .BYTE %01000000
  .BYTE %01000000
  .BYTE %11000001
  .BYTE %10000011
  .BYTE %00000010
  .BYTE %00000010
  .BYTE %00000000

		; Player Base Phase 0 (4 phase) (CHR 14)
  .BYTE %00000100  
  .BYTE %00001110  
  .BYTE %01111111
  .BYTE %11111111
  .BYTE %11111111
  .BYTE %11111111
  .BYTE %11111111
  .BYTE %11000000

  .BYTE %00000000  
  .BYTE %00000000  
  .BYTE %11000000  
  .BYTE %11100000
  .BYTE %11100000
  .BYTE %11100000
  .BYTE %11100000
  .BYTE %01100000

		; Player Base Phase 1
  .BYTE %00000001  
  .BYTE %00000011
  .BYTE %00011111
  .BYTE %00111111
  .BYTE %00111111
  .BYTE %00111111
  .BYTE %00111111
  .BYTE %00110000

  .BYTE %00000000  
  .BYTE %10000000  
  .BYTE %11110000
  .BYTE %11111000
  .BYTE %11111000
  .BYTE %11111000
  .BYTE %11111000
  .BYTE %00011000

		; Player Base Phase 2
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000111
  .BYTE %00001111
  .BYTE %00001111
  .BYTE %00001111
  .BYTE %00001111
  .BYTE %00001100

  .BYTE %01000000
  .BYTE %11100000
  .BYTE %11111100
  .BYTE %11111110
  .BYTE %11111110
  .BYTE %11111110
  .BYTE %11111110
  .BYTE %00000110

		; Player Base Phase 3
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000001
  .BYTE %00000011
  .BYTE %00000011
  .BYTE %00000011
  .BYTE %00000011
  .BYTE %00000011

  .BYTE %00010000
  .BYTE %00111000
  .BYTE %11111111
  .BYTE %11111111
  .BYTE %11111111
  .BYTE %11111111
  .BYTE %11111111
  .BYTE %00000001

  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %10000000
  .BYTE %10000000
  .BYTE %10000000
  .BYTE %10000000
  .BYTE %10000000

		; Player Missile Pos 0 (4 pos) (CHR 23)
  .BYTE %01000000  
  .BYTE %01000000  
  .BYTE %01000000  
  .BYTE %01000000  
  .BYTE %01000000  
  .BYTE %01000000  
  .BYTE %01000000  
  .BYTE %01000000  

		; Missile Pos 1 
  .BYTE %00010000  
  .BYTE %00010000  
  .BYTE %00010000  
  .BYTE %00010000  
  .BYTE %00010000  
  .BYTE %00010000  
  .BYTE %00010000  
  .BYTE %00010000  
		; Missile Pos 2
  .BYTE %00000100  
  .BYTE %00000100  
  .BYTE %00000100  
  .BYTE %00000100  
  .BYTE %00000100  
  .BYTE %00000100  
  .BYTE %00000100  
  .BYTE %00000100  
		; Missile Pos 3
  .BYTE %00000001
  .BYTE %00000001
  .BYTE %00000001
  .BYTE %00000001
  .BYTE %00000001
  .BYTE %00000001
  .BYTE %00000001
  .BYTE %00000001

		; Alien Bomb pos 0 (CHR 27)
  .BYTE %00100100
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00000000
  .BYTE %00000000

		; Bomb pos 1
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00100100
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000

		; Bomb pos 2
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00100100
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000

  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000

		; Bomb pos 3
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00100100
  .BYTE %00011000

  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000
  .BYTE %00000000

		; Chrset numbers 0-9 (CHR 33)
  .BYTE %00111100
  .BYTE %01100110
  .BYTE %01100110
  .BYTE %01100110
  .BYTE %01100110
  .BYTE %01100110
  .BYTE %00111100
  .BYTE %00000000

  .BYTE %00011000
  .BYTE %00111000
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00011000
  .BYTE %00111100
  .BYTE %00000000

  .BYTE %00111100
  .BYTE %01100110
  .BYTE %00000110
  .BYTE %00011100
  .BYTE %00110000
  .BYTE %01100000
  .BYTE %01111110
  .BYTE %00000000

  .BYTE %00111100
  .BYTE %01100110
  .BYTE %00000110
  .BYTE %00011100
  .BYTE %00000110
  .BYTE %01100110
  .BYTE %00111100
  .BYTE %00000000

  .BYTE %00000110
  .BYTE %00001110
  .BYTE %00011110
  .BYTE %00110110
  .BYTE %01100110
  .BYTE %01111111
  .BYTE %00000110
  .BYTE %00000000

  .BYTE %01111110
  .BYTE %01100000
  .BYTE %01111100
  .BYTE %00000110
  .BYTE %00000110
  .BYTE %01000110
  .BYTE %00111100
  .BYTE %00000000

  .BYTE %00111100
  .BYTE %01100110
  .BYTE %01100000
  .BYTE %01111100
  .BYTE %01100110
  .BYTE %01100110
  .BYTE %00111100
  .BYTE %00000000

  .BYTE %01111110
  .BYTE %00000110
  .BYTE %00001100
  .BYTE %00011000
  .BYTE %00110000
  .BYTE %00110000
  .BYTE %00110000
  .BYTE %00000000

  .BYTE %00111100
  .BYTE %01100110
  .BYTE %01100110
  .BYTE %00111100
  .BYTE %01100110
  .BYTE %01100110
  .BYTE %00111100
  .BYTE %00000000

  .BYTE %00111100
  .BYTE %01100110
  .BYTE %01100110
  .BYTE %00111110
  .BYTE %00000110
  .BYTE %01100110
  .BYTE %00111100
  .BYTE %00000000


ChrEnd:
FreeMem:
